@@ -22,7 +22,10 @@
 #  SOFTWARE.
 #  ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~
 
+import warnings
 from functools import wraps
+from typing import Callable
+from typing import Iterator
 from typing import Optional
 from typing import Sequence
 from typing import TypeVar
@@ -30,6 +33,7 @@ from typing import Union
 
 import numpy as np
 from torch.utils.data import Dataset
+from torch.utils.data import IterableDataset
 from torch.utils.data.dataloader import default_collate
 
 from disent.dataset.sampling import BaseDisentSampler
@@ -81,19 +85,19 @@ def wrapped_only(func):
 # ========================================================================= #
 
 
-_DO_COPY = object()
+_REF_ = object()
 
 
 class DisentDataset(Dataset, LengthIter):
 
-
     def __init__(
         self,
-        dataset: Union[Dataset, GroundTruthData],
+        dataset: Union[Dataset, GroundTruthData],  # TODO: this should be renamed to data
         sampler: Optional[BaseDisentSampler] = None,
-        transform=None,
-        augment=None,
-        return_indices: bool = False,
+        transform: Optional[callable] = None,
+        augment: Optional[callable] = None,
+        return_indices: bool = False,  # doesn't really hurt performance, might as well leave enabled by default?
+        return_factors: bool = False,
     ):
         super().__init__()
         # save attributes
@@ -102,22 +106,37 @@ class DisentDataset(Dataset, LengthIter):
         self._transform = transform
         self._augment = augment
         self._return_indices = return_indices
+        self._return_factors = return_factors
+        # check sampler
+        assert isinstance(self._sampler, BaseDisentSampler), f'{DisentDataset.__name__} got an invalid {BaseDisentSampler.__name__}: {type(self._sampler)}'
         # initialize sampler
         if not self._sampler.is_init:
             self._sampler.init(dataset)
+        # warn if we are overriding a transform
+        if self._transform is not None:
+            if hasattr(dataset, '_transform') and dataset._transform:
+                warnings.warn(f'{DisentDataset.__name__} has transform specified as well as wrapped dataset: {dataset}, are you sure this is intended?')
+        # check the dataset if we are returning the factors
+        if self._return_factors:
+            assert isinstance(self._dataset, GroundTruthData), f'If `return_factors` is `True`, then the dataset must be an instance of: {GroundTruthData.__name__}, got: {type(dataset)}'
 
     def shallow_copy(
         self,
-        transform=_DO_COPY,
-        augment=_DO_COPY,
-        return_indices=_DO_COPY,
+        dataset: Union[Dataset, GroundTruthData] =_REF_,  # TODO: this should be renamed to data
+        sampler: Optional[BaseDisentSampler] = _REF_,
+        transform: Optional[callable] = _REF_,
+        augment: Optional[callable] = _REF_,
+        return_indices: bool = _REF_,
+        return_factors: bool = _REF_,
     ) -> 'DisentDataset':
+        # instantiate shallow dataset copy, overwriting elements if specified
         return DisentDataset(
-            dataset=self._dataset,
-            sampler=self._sampler,
-            transform=self._transform if (transform is _DO_COPY) else transform,
-            augment=self._augment if (augment is _DO_COPY) else augment,
-            return_indices=self._return_indices if (return_indices is _DO_COPY) else return_indices,
+            dataset        = self._dataset               if (dataset is _REF_)        else dataset,
+            sampler        = self._sampler.uninit_copy() if (sampler is _REF_)        else sampler,
+            transform      = self._transform             if (transform is _REF_)      else transform,
+            augment        = self._augment               if (augment is _REF_)        else augment,
+            return_indices = self._return_indices        if (return_indices is _REF_) else return_indices,
+            return_factors = self._return_factors        if (return_factors is _REF_) else return_factors,
         )
 
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - #
@@ -128,6 +147,18 @@ class DisentDataset(Dataset, LengthIter):
     def data(self) -> Dataset:
         return self._dataset
 
+    @property
+    def sampler(self) -> BaseDisentSampler:
+        return self._sampler
+
+    @property
+    def transform(self) -> Optional[Callable[[object], object]]:
+        return self._transform
+
+    @property
+    def augment(self) -> Optional[Callable[[object], object]]:
+        return self._augment
+
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - #
     # Ground Truth Only                                                     #
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - #
@@ -176,15 +207,22 @@ class DisentDataset(Dataset, LengthIter):
         return self._dataset.gt_data
 
     @wrapped_only
-    def unwrapped_disent_dataset(self) -> 'DisentDataset':
-        sampler = self._sampler.uninit_copy()
-        assert type(sampler) is type(self._sampler)
-        return DisentDataset(
+    def unwrapped_shallow_copy(
+        self,
+        sampler: Optional[BaseDisentSampler] = _REF_,
+        transform: Optional[callable] = _REF_,
+        augment: Optional[callable] = _REF_,
+        return_indices: bool = _REF_,
+        return_factors: bool = _REF_,
+    ) -> 'DisentDataset':
+        # like shallow_copy, but unwrap the dataset instead!
+        return self.shallow_copy(
             dataset=self.wrapped_data,
             sampler=sampler,
-            transform=self._transform,
-            augment=self._augment,
-            return_indices=self._return_indices,
+            transform=transform,
+            augment=augment,
+            return_indices=return_indices,
+            return_factors=return_factors,
         )
 
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - #
@@ -275,6 +313,12 @@ class DisentDataset(Dataset, LengthIter):
         # add indices
         if self._return_indices:
             obs['idx'] = idxs
+        # add factors
+        if self._return_factors:
+            # >>> this is about 10% faster than below, because we do not need to do conversions!
+            obs['factors'] = tuple(np.array(np.unravel_index(idxs, self._dataset.factor_sizes)).T)
+            # >>> builtin but slower method, does some magic for more than 2 dims, could replace with faster try_njit method, but then we need numba!
+            # obs['factors1'] = tuple(self.gt_data.idx_to_pos(idxs))
         # done!
         return obs
 
@@ -285,38 +329,57 @@ class DisentDataset(Dataset, LengthIter):
     # TODO: default_collate should be replaced with a function
     #      that can handle tensors and nd.arrays, and return accordingly
 
-    def dataset_batch_from_indices(self, indices: Sequence[int], mode: str):
+    def dataset_batch_from_indices(self, indices: Sequence[int], mode: str, collate: bool = True):
         """Get a batch of observations X from a batch of factors Y."""
-        return default_collate([self.dataset_get(idx, mode=mode) for idx in indices])
+        batch = [self.dataset_get(idx, mode=mode) for idx in indices]
+        return default_collate(batch) if collate else batch
 
-    def dataset_sample_batch(self, num_samples: int, mode: str, replace: bool = False, return_indices: bool = False):
+    def dataset_sample_batch(self, num_samples: int, mode: str, replace: bool = False, return_indices: bool = False, collate: bool = True, seed: Optional[int] = None):
         """Sample a batch of observations X."""
         # built in np.random.choice cannot handle large values: https://github.com/numpy/numpy/issues/5299#issuecomment-497915672
-        indices = random_choice_prng(len(self), size=num_samples, replace=replace)
+        indices = random_choice_prng(len(self._dataset), size=num_samples, replace=replace, seed=seed)
         # return batch
-        batch = self.dataset_batch_from_indices(indices, mode=mode)
+        batch = self.dataset_batch_from_indices(indices, mode=mode, collate=collate)
         # return values
         if return_indices:
-            return batch, default_collate(indices)
+            return batch, (default_collate(indices) if collate else indices)
         else:
             return batch
 
+    def dataset_sample_elems(self, num_samples: int, mode: str, return_indices: bool = False, seed: Optional[int] = None):
+        """Sample uncollated elements with replacement, like `dataset_sample_batch`"""
+        return self.dataset_sample_batch(num_samples=num_samples, mode=mode, replace=True, return_indices=return_indices, collate=False, seed=seed)
+
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - #
     # Batches -- Ground Truth Only                                          #
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - #
 
     @groundtruth_only
-    def dataset_batch_from_factors(self, factors: np.ndarray, mode: str):
+    def dataset_batch_from_factors(self, factors: np.ndarray, mode: str, collate: bool = True):
         """Get a batch of observations X from a batch of factors Y."""
         indices = self.gt_data.pos_to_idx(factors)
-        return self.dataset_batch_from_indices(indices, mode=mode)
+        return self.dataset_batch_from_indices(indices, mode=mode, collate=collate)
 
     @groundtruth_only
-    def dataset_sample_batch_with_factors(self, num_samples: int, mode: str):
+    def dataset_sample_batch_with_factors(self, num_samples: int, mode: str, collate: bool = True):
         """Sample a batch of observations X and factors Y."""
         factors = self.gt_data.sample_factors(num_samples)
-        batch = self.dataset_batch_from_factors(factors, mode=mode)
-        return batch, default_collate(factors)
+        batch = self.dataset_batch_from_factors(factors, mode=mode, collate=collate)
+        return batch, (default_collate(factors) if collate else factors)
+
+
+class DisentIterDataset(IterableDataset, DisentDataset):
+
+    # make sure we cannot obtain the length directly
+    __len__ = None
+
+    def __iter__(self):
+        # this takes priority over __getitem__, otherwise __getitem__ would need to
+        # raise an IndexError if out of bounds to signal the end of iteration
+        while True:
+            # yield the entire dataset
+            # - repeating when it is done!
+            yield from (self[i] for i in range(len(self._dataset)))
 
 
 # ========================================================================= #
