@@ -86,6 +86,7 @@ if __name__ == '__main__':
 
     def main(progress=True, num_workers=0, batch_size=256):  # try changing workers to zero on MacOS
         from disent.dataset import data
+        from docs.examples.extend_experiment.code import groundtruth__xyblocks as edat
         from disent.dataset.transform import ToImgTensorF32
 
         for data_cls in [
@@ -99,6 +100,20 @@ if __name__ == '__main__':
             # groundtruth -- impl synthetic
             data.XYObjectData,
             data.XYObjectShadedData,
+            edat.XYBlocksData,
+            data.XYSquaresData,
+            data.XYSquaresMinimalData,
+            data.XColumnsData,
+            # groundtruth -- increasing overlap
+            (data.XYSquaresData, dict(grid_size=8, grid_spacing=8)),
+            (data.XYSquaresData, dict(grid_size=8, grid_spacing=7)),
+            (data.XYSquaresData, dict(grid_size=8, grid_spacing=6)),
+            (data.XYSquaresData, dict(grid_size=8, grid_spacing=5)),
+            (data.XYSquaresData, dict(grid_size=8, grid_spacing=4)),
+            (data.XYSquaresData, dict(grid_size=8, grid_spacing=3)),
+            (data.XYSquaresData, dict(grid_size=8, grid_spacing=2)),
+            (data.XYSquaresData, dict(grid_size=8, grid_spacing=1)),
+            (data.XYSquaresData, dict(rgb=False)),
             # large datasets
             (data.Mpi3dData, dict(subset='toy',       in_memory=True)),
             (data.Mpi3dData, dict(subset='realistic', in_memory=True)),
